@@ -2,7 +2,7 @@
 //! that contains Messages instead of Interactions.
 
 use std::cmp::Ordering;
-use std::collections::BTreeMap;
+use std::collections::{BTreeMap, HashMap};
 use std::fs;
 use std::fs::File;
 use std::io::prelude::*;
@@ -14,8 +14,7 @@ use itertools::EitherOrBoth::{Both, Left, Right};
 use itertools::Itertools;
 use log::*;
 use maplit::*;
-use serde::{Deserialize, Serialize};
-use serde_json::{json, Value};
+use serde_json::{json, Value, Map};
 
 use crate::{Consumer, PactSpecification, Provider};
 #[cfg(not(target_family = "wasm"))] use crate::file_utils::with_read_lock;
@@ -23,15 +22,16 @@ use crate::{Consumer, PactSpecification, Provider};
 use crate::interaction::Interaction;
 use crate::message::Message;
 use crate::pact::{determine_spec_version, Pact, parse_meta_data, ReadWritePact};
-use crate::sync_pact::RequestResponsePact;
-use crate::verify_json::{json_type_of, PactFileVerificationResult, PactJsonVerifier, ResultLevel};
-use crate::v4::pact::V4Pact;
 use crate::PACT_RUST_VERSION;
+use crate::plugins::PluginData;
+use crate::sync_pact::RequestResponsePact;
+use crate::v4::pact::V4Pact;
+use crate::verify_json::{json_type_of, PactFileVerificationResult, PactJsonVerifier, ResultLevel};
 
 /// Struct that represents a pact between the consumer and provider of a service.
 /// It contains a list of Messages instead of Interactions, but is otherwise
 /// identical to `struct Pact`.
-#[derive(Debug, Clone, Deserialize, Serialize, PartialEq)]
+#[derive(Debug, Clone, PartialEq)]
 pub struct MessagePact {
     /// Consumer side of the pact
     pub consumer: Consumer,
@@ -54,8 +54,8 @@ impl Pact for MessagePact {
     self.provider.clone()
   }
 
-  fn interactions(&self) -> Vec<&dyn Interaction> {
-    self.messages.iter().map(|i| i as &dyn Interaction).collect()
+  fn interactions(&self) -> Vec<Box<dyn Interaction + Send + Sync>> {
+    self.messages.iter().map(|i| i.boxed()).collect()
   }
 
   fn metadata(&self) -> BTreeMap<String, BTreeMap<String, String>> {
@@ -95,7 +95,8 @@ impl Pact for MessagePact {
       consumer: self.consumer.clone(),
       provider: self.provider.clone(),
       interactions,
-      metadata: self.metadata.iter().map(|(k, v)| (k.clone(), json!(v))).collect()
+      metadata: self.metadata.iter().map(|(k, v)| (k.clone(), json!(v))).collect(),
+      .. V4Pact::default()
     })
   }
 
@@ -103,11 +104,11 @@ impl Pact for MessagePact {
     self.specification_version.clone()
   }
 
-  fn boxed(&self) -> Box<dyn Pact + Send> {
+  fn boxed(&self) -> Box<dyn Pact + Send + Sync> {
     Box::new(self.clone())
   }
 
-  fn arced(&self) -> Arc<dyn Pact + Send> {
+  fn arced(&self) -> Arc<dyn Pact + Send + Sync> {
     Arc::new(self.clone())
   }
 
@@ -124,6 +125,37 @@ impl Pact for MessagePact {
       }
     }
   }
+
+  fn requires_plugins(&self) -> bool {
+    false
+  }
+
+  fn plugin_data(&self) -> Vec<PluginData> {
+    Vec::default()
+  }
+
+  fn is_v4(&self) -> bool {
+    false
+  }
+
+  fn add_plugin(
+    &mut self,
+    _name: &str,
+    _version: &str,
+    _plugin_data: Option<HashMap<String, Value>>
+  ) -> anyhow::Result<()> {
+    Err(anyhow!("Plugins can only be used with V4 format pacts"))
+  }
+
+  fn add_md_version(&mut self, key: &str, version: &str) {
+    if let Some(md) = self.metadata.get_mut("pactRust") {
+      md.insert(key.to_string(), version.to_string());
+    } else {
+      self.metadata.insert("pactRust".to_string(), btreemap! {
+        key.to_string() => version.to_string()
+      });
+    }
+  }
 }
 
 impl MessagePact {
@@ -188,9 +220,11 @@ impl MessagePact {
         md_map.insert(
             "pactSpecification".to_string(),
             json!({"version" : pact_spec.version_str()}));
-        md_map.insert(
-            "pactRust".to_string(),
-            json!({"version" : PACT_RUST_VERSION.unwrap_or("unknown").to_string()}));
+        let version_entry = md_map.entry("pactRust".to_string())
+          .or_insert(Value::Object(Map::default()));
+        if let Value::Object(map) = version_entry {
+          map.insert("models".to_string(), Value::String(PACT_RUST_VERSION.unwrap_or("unknown").to_string()));
+        }
         md_map
     }
 
@@ -223,10 +257,7 @@ impl MessagePact {
         let mut file = File::create(path)?;
 
         file.write_all(
-            format!("{}",
-                serde_json::to_string_pretty(
-                    &self.to_json(pact_spec)?)?
-            ).as_bytes()
+          serde_json::to_string_pretty(&self.to_json(pact_spec)?)?.as_bytes()
         )?;
 
         Ok(())
@@ -267,7 +298,7 @@ impl ReadWritePact for MessagePact {
     })
   }
 
-  fn merge(&self, pact: &dyn Pact) -> anyhow::Result<Box<dyn Pact>> {
+  fn merge(&self, pact: &dyn Pact) -> anyhow::Result<Box<dyn Pact + Send + Sync>> {
     if self.consumer.name == pact.consumer().name && self.provider.name == pact.provider().name {
       let messages: Vec<Result<Message, String>> = self.messages.iter()
         .merge_join_by(pact.interactions().iter(), |a, b| {
@@ -299,7 +330,7 @@ impl ReadWritePact for MessagePact {
             .filter(|i| i.is_ok())
             .map(|i| i.as_ref().unwrap().clone()).collect(),
           metadata: self.metadata.clone(),
-          specification_version: self.specification_version.clone()
+          specification_version: self.specification_version
         }))
       } else {
         Err(anyhow!("Unable to merge pacts: {}", errors.join(", ")))
@@ -334,6 +365,7 @@ impl PactJsonVerifier for MessagePact {
 mod tests {
   use expectest::expect;
   use expectest::prelude::*;
+  use pretty_assertions::assert_eq;
 
   use super::*;
 
@@ -516,6 +548,11 @@ mod tests {
         expect!(pact.as_ref()).to(be_ok());
         let pact = pact.unwrap();
         let contents = pact.to_json(PactSpecification::V3);
-        expect!(contents.unwrap().to_string()).to(be_equal_to("{\"consumer\":{\"name\":\"Consumer\"},\"messages\":[{\"contents\":{\"hello\":\"world\"},\"description\":\"Message Description\",\"metadata\":{\"contentType\":\"application/json\"}}],\"metadata\":{\"pactRust\":{\"version\":\"".to_owned() + env!("CARGO_PKG_VERSION") + "\"},\"pactSpecification\":{\"version\":\"3.0.0\"}},\"provider\":{\"name\":\"Alice Service\"}}"));
+        assert_eq!(contents.unwrap().to_string(),
+          "{\"consumer\":{\"name\":\"Consumer\"},\"messages\":[\
+          {\"contents\":{\"hello\":\"world\"},\"description\":\"Message Description\",\
+          \"metadata\":{\"contentType\":\"application/json\"}}],\
+          \"metadata\":{\"pactRust\":{\"models\":\"".to_owned() + env!("CARGO_PKG_VERSION") +
+            "\"},\"pactSpecification\":{\"version\":\"3.0.0\"}},\"provider\":{\"name\":\"Alice Service\"}}");
     }
 }
