@@ -9,25 +9,25 @@ use std::sync::{Arc, Mutex};
 use anyhow::anyhow;
 use itertools::Itertools;
 use log::*;
-use serde_json::{json, Value};
+use serde_json::{json, Map, Value};
 
 use crate::bodies::OptionalBody;
 use crate::content_types::ContentType;
 use crate::interaction::Interaction;
-use crate::json_utils::json_to_string;
+use crate::json_utils::{is_empty, json_to_string};
 use crate::matchingrules::MatchingRules;
 use crate::message::Message;
 use crate::provider_states::ProviderState;
 use crate::sync_interaction::RequestResponseInteraction;
 use crate::v4::async_message::AsynchronousMessage;
-use crate::v4::interaction::V4Interaction;
+use crate::v4::interaction::{InteractionMarkup, parse_plugin_config, V4Interaction};
 use crate::v4::message_parts::MessageContents;
 use crate::v4::synch_http::SynchronousHttp;
 use crate::v4::V4InteractionType;
 
 /// Synchronous interactions as a request message to a sequence of response messages
 #[derive(Debug, Clone, Eq)]
-pub struct SynchronousMessages {
+pub struct SynchronousMessage {
   /// Interaction ID. This will only be set if the Pact file was fetched from a Pact Broker
   pub id: Option<String>,
   /// Unique key for this interaction
@@ -45,10 +45,16 @@ pub struct SynchronousMessages {
   pub response: Vec<MessageContents>,
 
   /// If this interaction is pending. Pending interactions will never fail the build if they fail
-  pub pending: bool
+  pub pending: bool,
+
+  /// Configuration added by plugins
+  pub plugin_config: HashMap<String, HashMap<String, Value>>,
+
+  /// Text markup to use to render the interaction in a UI
+  pub interaction_markup: InteractionMarkup
 }
 
-impl SynchronousMessages {
+impl SynchronousMessage {
   fn calc_hash(&self) -> String {
     let mut s = DefaultHasher::new();
     self.hash(&mut s);
@@ -56,15 +62,15 @@ impl SynchronousMessages {
   }
 
   /// Creates a new version with a calculated key
-  pub fn with_key(&self) -> SynchronousMessages {
-    SynchronousMessages {
+  pub fn with_key(&self) -> SynchronousMessage {
+    SynchronousMessage {
       key: Some(self.calc_hash()),
       .. self.clone()
     }
   }
 
   /// Parse the JSON into a SynchronousMessages structure
-  pub fn from_json(json: &Value, index: usize) -> anyhow::Result<SynchronousMessages> {
+  pub fn from_json(json: &Value, index: usize) -> anyhow::Result<SynchronousMessage> {
     if json.is_object() {
       let id = json.get("_id").map(|id| json_to_string(id));
       let key = json.get("key").map(|id| json_to_string(id));
@@ -75,6 +81,7 @@ impl SynchronousMessages {
         },
         None => format!("Interaction {}", index)
       };
+
       let comments = match json.get("comments") {
         Some(v) => match v {
           Value::Object(map) => map.iter()
@@ -86,17 +93,23 @@ impl SynchronousMessages {
         },
         None => Default::default()
       };
+
       let provider_states = ProviderState::from_json(json);
       let request = json.get("request")
-        .ok_or(anyhow!("JSON for SynchronousMessages does not contain a 'request' object"))?;
+        .ok_or_else(|| anyhow!("JSON for SynchronousMessages does not contain a 'request' object"))?;
       let response = json.get("response")
-        .ok_or(anyhow!("JSON for SynchronousMessages does not contain a 'response' array"))?
+        .ok_or_else(|| anyhow!("JSON for SynchronousMessages does not contain a 'response' array"))?
         .as_array()
-        .ok_or(anyhow!("JSON for SynchronousMessages does not contain a 'response' array"))?;
+        .ok_or_else(|| anyhow!("JSON for SynchronousMessages does not contain a 'response' array"))?;
       let responses =
         response.iter()
           .map(|message| MessageContents::from_json(message))
           .collect::<Vec<anyhow::Result<MessageContents>>>();
+
+      let plugin_config = parse_plugin_config(json);
+      let interaction_markup = json.get("interactionMarkup")
+        .map(|markup| InteractionMarkup::from_json(markup)).unwrap_or_default();
+
       if responses.iter().any(|res| res.is_err()) {
         let errors = responses.iter()
           .filter(|res| res.is_err())
@@ -104,7 +117,7 @@ impl SynchronousMessages {
           .join(", ");
         Err(anyhow!("Failed to parse SynchronousMessages responses - {}", errors))
       } else {
-        Ok(SynchronousMessages {
+        Ok(SynchronousMessage {
           id,
           key,
           description,
@@ -113,7 +126,9 @@ impl SynchronousMessages {
           request: MessageContents::from_json(request)?,
           response: responses.iter().map(|res| res.as_ref().unwrap().clone()).collect(),
           pending: json.get("pending")
-            .map(|value| value.as_bool().unwrap_or_default()).unwrap_or_default()
+            .map(|value| value.as_bool().unwrap_or_default()).unwrap_or_default(),
+          plugin_config,
+          interaction_markup
         })
       }
     } else {
@@ -122,7 +137,7 @@ impl SynchronousMessages {
   }
 }
 
-impl V4Interaction for SynchronousMessages {
+impl V4Interaction for SynchronousMessage {
   fn to_json(&self) -> Value {
     let mut json = json!({
       "type": V4InteractionType::Synchronous_Messages.to_string(),
@@ -139,10 +154,26 @@ impl V4Interaction for SynchronousMessages {
         self.provider_states.iter().map(|p| p.to_json()).collect()));
     }
 
-    if !self.comments.is_empty() {
+    let comments: Map<String, Value> = self.comments.iter()
+      .filter(|(_k, v)| !is_empty(v))
+      .map(|(k, v)| (k.clone(), v.clone()))
+      .collect();
+    if !comments.is_empty() {
       let map = json.as_object_mut().unwrap();
-      map.insert("comments".to_string(), self.comments.iter()
-        .map(|(k, v)| (k.clone(), v.clone())).collect());
+      map.insert("comments".to_string(), Value::Object(comments));
+    }
+
+    if !self.plugin_config.is_empty() {
+      let map = json.as_object_mut().unwrap();
+      map.insert("pluginConfiguration".to_string(), self.plugin_config.iter()
+        .map(|(k, v)|
+          (k.clone(), Value::Object(v.iter().map(|(k, v)| (k.clone(), v.clone())).collect()))
+        ).collect());
+    }
+
+    if !self.interaction_markup.is_empty() {
+      let map = json.as_object_mut().unwrap();
+      map.insert("interactionMarkup".to_string(), self.interaction_markup.to_json());
     }
 
     json
@@ -171,9 +202,25 @@ impl V4Interaction for SynchronousMessages {
   fn v4_type(&self) -> V4InteractionType {
     V4InteractionType::Synchronous_Messages
   }
+
+  fn plugin_config(&self) -> HashMap<String, HashMap<String, Value>> {
+    self.plugin_config.clone()
+  }
+
+  fn plugin_config_mut(&mut self) -> &mut HashMap<String, HashMap<String, Value>> {
+    &mut self.plugin_config
+  }
+
+  fn interaction_markup(&self) -> InteractionMarkup {
+    self.interaction_markup.clone()
+  }
+
+  fn interaction_markup_mut(&mut self) -> &mut InteractionMarkup {
+    &mut self.interaction_markup
+  }
 }
 
-impl Interaction for SynchronousMessages {
+impl Interaction for SynchronousMessage {
   fn type_of(&self) -> String {
     format!("V4 {}", self.v4_type())
   }
@@ -202,10 +249,22 @@ impl Interaction for SynchronousMessages {
     self.description.clone()
   }
 
+  fn set_id(&mut self, id: Option<String>) {
+    self.id = id;
+  }
+
+  fn set_description(&mut self, description: &str) {
+    self.description = description.to_string();
+  }
+
   fn provider_states(&self) -> Vec<ProviderState> {
     self.provider_states.clone()
   }
 
+  fn provider_states_mut(&mut self) -> &mut Vec<ProviderState> {
+    &mut self.provider_states
+  }
+
   fn contents(&self) -> OptionalBody {
     OptionalBody::Missing
   }
@@ -226,6 +285,10 @@ impl Interaction for SynchronousMessages {
     Some(self.boxed_v4())
   }
 
+  fn as_v4_mut(&mut self) -> Option<&mut dyn V4Interaction> {
+    Some(self)
+  }
+
   fn as_v4_http(&self) -> Option<SynchronousHttp> {
     None
   }
@@ -234,15 +297,31 @@ impl Interaction for SynchronousMessages {
     None
   }
 
-  fn as_v4_sync_message(&self) -> Option<SynchronousMessages> {
+  fn as_v4_sync_message(&self) -> Option<SynchronousMessage> {
     Some(self.clone())
   }
 
-  fn boxed(&self) -> Box<dyn Interaction + Send> {
+  fn as_v4_http_mut(&mut self) -> Option<&mut SynchronousHttp> {
+    None
+  }
+
+  fn is_v4_sync_message(&self) -> bool {
+    true
+  }
+
+  fn as_v4_async_message_mut(&mut self) -> Option<&mut AsynchronousMessage> {
+    None
+  }
+
+  fn as_v4_sync_message_mut(&mut self) -> Option<&mut SynchronousMessage> {
+    Some(self)
+  }
+
+  fn boxed(&self) -> Box<dyn Interaction + Send + Sync> {
     Box::new(self.clone())
   }
 
-  fn arced(&self) -> Arc<dyn Interaction + Send> {
+  fn arced(&self) -> Arc<dyn Interaction + Send + Sync> {
     Arc::new(self.clone())
   }
 
@@ -259,9 +338,9 @@ impl Interaction for SynchronousMessages {
   }
 }
 
-impl Default for SynchronousMessages {
+impl Default for SynchronousMessage {
   fn default() -> Self {
-    SynchronousMessages {
+    SynchronousMessage {
       id: None,
       key: None,
       description: "Synchronous/Message Interaction".to_string(),
@@ -269,12 +348,14 @@ impl Default for SynchronousMessages {
       comments: Default::default(),
       request: Default::default(),
       response: Default::default(),
-      pending: false
+      pending: false,
+      plugin_config: Default::default(),
+      interaction_markup: Default::default()
     }
   }
 }
 
-impl PartialEq for SynchronousMessages {
+impl PartialEq for SynchronousMessage {
   fn eq(&self, other: &Self) -> bool {
     self.description == other.description && self.provider_states == other.provider_states &&
       self.request == other.request && self.response == other.response &&
@@ -282,7 +363,7 @@ impl PartialEq for SynchronousMessages {
   }
 }
 
-impl Hash for SynchronousMessages {
+impl Hash for SynchronousMessage {
   fn hash<H: Hasher>(&self, state: &mut H) {
     self.description.hash(state);
     self.provider_states.hash(state);
@@ -292,7 +373,7 @@ impl Hash for SynchronousMessages {
   }
 }
 
-impl Display for SynchronousMessages {
+impl Display for SynchronousMessage {
   fn fmt(&self, f: &mut Formatter) -> std::fmt::Result {
     let pending = if self.pending { " [PENDING]" } else { "" };
     write!(f, "V4 Synchronous Message Interaction{} ( id: {:?}, description: \"{}\", provider_states: {:?}, request: {}, response: {:?} )",
